Class: NA=ArgumentError.new
Overview
N/A exception is useful for specifying non-argument defaults when nil is a valid value.
def f(x=NA)
end
NA is also used to represent an argument “slot” for Proc#partial.
NA is an instance of ArgumentError.
N/A exception is useful for specifying non-argument defaults when nil is a valid value.
def f(x=NA)
end
NA is also used to represent an argument “slot” for Proc#partial.
NA is an instance of ArgumentError.